////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   :  "   API"
//  * Details   : http://alexander-bagel.blogspot.ru/2013/01/intercept.html
//  * Unit Name : CommonHotPatch
//  * Purpose   :     HotPatch
//  * Author    :  (Rouse_) 
//  * Version   : 1.0.1
//  * Home Page : http://rouse.drkb.ru
//  * Home Blog : http://alexander-bagel.blogspot.com/
//  ****************************************************************************
//

unit CommonHotPatch;

interface

uses
  Windows;

const
  LOCK_JMP_OPKODE: Word = $F9EB;  

type
  //      JMP NEAR OFFSET
  TNearJmpSpliceRec = packed record
    JmpOpcode: Byte;
    Offset: DWORD;
  end;

  THotPachSpliceData = packed record
    FuncAddr: FARPROC;
    SpliceRec: TNearJmpSpliceRec;
    LockJmp: Word;
  end;

var
  HotPathSpliceRec: THotPachSpliceData;

  procedure InitHotPatchSpliceRec;
  procedure SpliceNearJmp(FuncAddr: Pointer; NewData: TNearJmpSpliceRec);
  procedure SpliceLockJmp(FuncAddr: Pointer; NewData: Word);

implementation

//        
procedure SpliceNearJmp(FuncAddr: Pointer; NewData: TNearJmpSpliceRec);
var
  OldProtect: DWORD;
begin
  VirtualProtect(FuncAddr, SizeOf(TNearJmpSpliceRec),
    PAGE_EXECUTE_READWRITE, OldProtect);
  try
    Move(NewData, FuncAddr^, SizeOf(TNearJmpSpliceRec));
  finally
    VirtualProtect(FuncAddr, SizeOf(TNearJmpSpliceRec), OldProtect, OldProtect);
    FlushInstructionCache(GetCurrentProcess, FuncAddr, SizeOf(TNearJmpSpliceRec));
  end;
end;

//        
procedure SpliceLockJmp(FuncAddr: Pointer; NewData: Word);
var
  OldProtect: DWORD;
begin
  VirtualProtect(FuncAddr, 2, PAGE_EXECUTE_READWRITE, OldProtect);
  try
    asm
      mov  ax, NewData
      mov  ecx, FuncAddr
      lock xchg word ptr [ecx], ax
    end; 
  finally
    VirtualProtect(FuncAddr, 2, OldProtect, OldProtect);
    FlushInstructionCache(GetCurrentProcess, FuncAddr, 2);
  end;
end;

function InterceptedMessageBoxA(Wnd: HWND; lpText, lpCaption: PAnsiChar; uType: UINT): Integer; stdcall;
var
  S: AnsiString;
begin
  //  
  SpliceLockJmp(HotPathSpliceRec.FuncAddr, HotPathSpliceRec.LockJmp);
  try
    //   
    S := AnsiString('Function interepted. Original message: ' + lpText);
    Result := MessageBoxA(Wnd, PAnsiChar(S), lpCaption, uType);
  finally
    //  
    SpliceLockJmp(HotPathSpliceRec.FuncAddr, LOCK_JMP_OPKODE);
  end;
end;

procedure InitHotPatchSpliceRec;
begin
  //     
  HotPathSpliceRec.FuncAddr :=
    GetProcAddress(GetModuleHandle(user32), 'MessageBoxA');
  //      ,    
  Move(HotPathSpliceRec.FuncAddr^, HotPathSpliceRec.LockJmp, 2);
  //   JMP NEAR
  HotPathSpliceRec.SpliceRec.JmpOpcode := $E9;
  //   
  HotPathSpliceRec.SpliceRec.Offset :=
    PAnsiChar(@InterceptedMessageBoxA) + 5 -
    PAnsiChar(HotPathSpliceRec.FuncAddr) - SizeOf(TNearJmpSpliceRec);
end;

end.
